package SIM;

public class Sim2 extends Sim{
	
	public static double getDeltaH(double thi, double Wi, double tho, double Wo){
		double h_o, h_i;
		h_o = 1.007 * tho + Wo * (2501 + 1.86 * tho);
		h_i = 1.007 * thi + Wi * (2501 + 1.86 * thi);
		return  h_o - h_i;
	}
	
	@Override
	public boolean run(){
		double c_w = 4.2; // kJ/kg-K
		double c_dl = 2.1; // kJ/kg-K
		double Q_AD = 0.0; // C
		double Q_AG = 0.0; // C
		double Q_GR = 0.0; // C
		double Q_AR = 0.0; // C
	    double Q_AK = 0.0; // C
	    double Q_DK = 0.0; // C
	    double C_w, C_s, C_t, C_x, C_dl;
	    double C_minK, C_minG, C_minD, C_minR;
	    double M_minD, M_minR;
		C_w = c_w * getN_wKi().getMFlow();
		C_s = c_w * getN_sGi().getMFlow();
		C_t = 1.04 * getN_tDi().getMFlow();
		C_x = 1.04 * getN_xRi().getMFlow();
		C_dl = c_dl * getN_cDi().getMFlow();
		C_minK = Math.min(C_w, C_dl);
		C_minG = Math.min(C_s, C_dl);
		C_minD = Math.min(C_t, C_dl);
		C_minR = Math.min(C_x, C_dl);
		M_minD = Math.min(getN_tDi().getMFlow(), getN_cDi().getMFlow());
		M_minR = Math.min(getN_xRi().getMFlow(), getN_cDi().getMFlow());
		int k = 0;
		double delta1 = 100;
		double delta2 = 100;
		double ref1, ref2;

		/* Primero asumimos que las temperaturas de entrada y salida en K
		 * son iguales a la temperatura de entrada del agua potable
		 */
		ref1 = getN_wKi().getTem(); ref2 = getN_cDi().getXCon();
		getN_cDi().setTem(ref1); getN_cKo().setTem(ref1); getN_cAi().setTem(ref1);
		getN_cKi().setTem(ref1); getN_tDo().setTem(ref1);
		getN_cRo().setTem(ref1); getN_xRo().setTem(ref1);
		getN_cRo().setXCon(ref2); getN_hDo().setXCon(ref2); getN_tDo().setXCon(ref2);
		getN_hRi().setXCon(ref2); getN_xRo().setXCon(ref2);
		// Comienza el ciclo Gauss-Seidel
		System.out.println("n\tT_tDo\tW_tDo\tT_cDi\tX_cDi\tT_hDo\tX_hDo\tT_hRi" 
				+ "\tX_hRi\tT_cRo\tX_cRo\tT_xRo\tW_xRo\tdelta");
		while (k < 100 && (delta1 > 0.001 || delta2 > 1E-5)){
			// Tubera de K a D
			getN_cDi().setTem(getN_cKo().getTem() + Q_DK / getN_cDi().getMFlow() / c_dl);
			getN_cDi().setXCon(getN_cRo().getXCon());
			// Salida del aire tratado por el deshumidificador
			getN_tDo().setTem(getN_tDi().getTem() - D.getEffS() * C_minD / C_t *(getN_tDi().getTem() - getN_cDi().getTem()));
			getN_tDo().setXCon(getN_tDi().getXCon() - D.getEffL() * M_minD / getN_tDi().getMFlow() *(getN_tDi().getXCon() - getN_cDi().getXCon()));
			// Salida del lquido desecante del deshumidificador
			getN_hDo().setTem(getN_cDi().getTem() + getDeltaH(getN_tDi().getTem(), getN_tDi().getXCon(), getN_tDo().getTem(), getN_tDo().getXCon()) / getN_cDi().getMFlow() / c_dl);
			getN_hDo().setXCon(getN_cDi().getXCon() + getN_tDi().getMFlow() / getN_cDi().getMFlow() * (getN_tDi().getXCon() - getN_tDo().getXCon()));
			// Tubera de D a A
			getN_hAi().setTem(getN_hDo().getTem() + Q_AD / getN_cDi().getMFlow() / c_dl);
			// Intercambiador A en lnea h
			getN_hAo().setTem(getN_hAi().getTem() - A.getEffS() * (getN_hAi().getTem() - getN_cAi().getTem()));
			// Tubera de A a G
			getN_hGi().setTem(getN_hAo().getTem() + Q_AG / getN_cDi().getMFlow() / c_dl);
			// Intercambiador G
			getN_hGo().setTem(getN_hGi().getTem() + G.getEffS() * C_minG / C_dl * (getN_sGi().getTem() - getN_hGi().getTem()));
			getN_sGo().setTem(getN_sGi().getTem() - G.getEffS() * C_minG / C_s * (getN_sGi().getTem() - getN_hGi().getTem()));
			// Tubera de G a R
			getN_hRi().setTem(getN_hGo().getTem() + Q_GR / getN_cDi().getMFlow() / c_dl);
			getN_hRi().setXCon(getN_hDo().getXCon());
			// Salida del aire tratado por el regenerador
			getN_xRo().setTem(getN_xRi().getTem() - R.getEffS() * C_minR / C_x * (getN_xRi().getTem() - getN_hRi().getTem()));
			getN_xRo().setXCon(getN_xRi().getXCon() - R.getEffL() * M_minR / getN_xRi().getMFlow() * (getN_xRi().getXCon() - getN_hRi().getXCon()));
			// Salida del lquido desecante del regenerador	
			getN_cRo().setTem(getN_hRi().getTem() + getDeltaH(getN_xRi().getTem(), getN_xRi().getXCon(), getN_xRo().getTem(), getN_xRo().getXCon()) / getN_cDi().getMFlow() / c_dl);
			getN_cRo().setXCon(getN_hRi().getXCon() + getN_xRi().getMFlow() / getN_cDi().getMFlow() * (getN_xRi().getXCon() - getN_xRo().getXCon()));
			// Tubera de R a A
			getN_cAi().setTem(getN_cRo().getTem() + Q_AR / getN_cDi().getMFlow() / c_dl);
			// Intercambiador A por la lnea c
			getN_cAo().setTem(getN_cAi().getTem() - A.getEffS() * (getN_cAi().getTem() - getN_hAi().getTem()));
			// Tubera de A a K
			getN_cKi().setTem(getN_cAo().getTem() + Q_AK / getN_cDi().getMFlow() / c_dl);
			// Intercambiador K
			getN_cKo().setTem(getN_cKi().getTem() - K.getEffS() * C_minK / C_dl * (getN_cKi().getTem() - getN_wKi().getTem()));
			getN_wKo().setTem(getN_wKi().getTem() + K.getEffS() * C_minK / C_w * (getN_cKi().getTem() - getN_wKi().getTem()));
			// Termina el ciclo y se compara la referencia
			System.out.println(String.valueOf(k) + "\t" + String.format("%.2f", getN_tDo().getTem())
				+ "\t" + String.format("%.2f", getN_tDo().getXCon() * 1000) + "\t" + String.format("%.2f", getN_cDi().getTem())
				+ "\t" + String.format("%.2f", getN_cDi().getXCon() * 1000) + "\t" + String.format("%.2f", getN_hDo().getTem()) 
				+ "\t" + String.format("%.2f", getN_hDo().getXCon() * 1000) + "\t" + String.format("%.2f", getN_hRi().getTem())
				+ "\t" + String.format("%.2f", getN_hRi().getXCon() * 1000) + "\t" + String.format("%.2f", getN_cRo().getTem())
				+ "\t" + String.format("%.2f", getN_cRo().getXCon() * 1000) + "\t" + String.format("%.2f", getN_xRo().getTem())
				+ "\t" + String.format("%.2f", getN_xRo().getXCon() * 1000) + "\t" + String.format("%.5f", delta2));
			k++;
			delta1 = Math.abs(getN_cKo().getTem() - ref1);
			delta2 = Math.abs(getN_cDi().getXCon() - ref2);
			ref1 = getN_cKo().getTem(); ref2 = getN_cDi().getXCon();
		}
		return (k < 100) ? true : false;
	}
	
	public static void main(String[] args){
		// Input parameters
		Sim2 sim = new Sim2();
		sim.getN_tDi().setTem(20.0); // C
		sim.getN_xRi().setTem(25.0); // C
		sim.getN_wKi().setTem(15.0); // C
		sim.getN_sGi().setTem(70.0); // C
		sim.getN_tDi().setXCon(0.012); // kg/kg
		sim.getN_xRi().setXCon(0.012); // kg/kg
		sim.getN_cDi().setXCon(0.01); // kg/kg
		sim.getN_tDi().setMFlow(0.5); // kg/s
		sim.getN_xRi().setMFlow(0.5); // kg/s
		sim.getN_cDi().setMFlow(1.0); // kg/s
		sim.getN_wKi().setMFlow(1.0); // kg/s
		sim.getN_sGi().setMFlow(1.0); // kg/s
		sim.setEp_A(0.8);
		sim.setEp_K(0.8);
		sim.setEp_G(0.8);
		sim.setEp_D(0.8, 0.8);
		sim.setEp_R(0.8, 0.8);
		sim.run();
	}
}



