package SIM;

import Elements.*;

public class Sim3 extends Sim{
	
	public Sim3(){
		super();
	}
	
	@Override
	public boolean run(){
		int k = 0;
		int max = 10000;
		double delta1 = 100;
		double delta2 = 100;
		Node ref;
		try{
		/* Primero asumimos que las temperaturas de entrada y salida en K
		 * son iguales a la temperatura de entrada del agua potable
		 */
		ref = new Node();
		ref.setTem(getN_cKi().getTem());
		ref.setXCon(getN_cKi().getXCon());
		setN_cAi(getN_cKi()); 
		setN_cKo(getN_cKi()); 
		setN_cRo(getN_cKi());
		// Comienza el ciclo Gauss-Seidel
		System.out.println("n\tT_tDo\tW_tDo\tT_cDi\tX_cDi\tT_hDo\tX_hDo\tT_hRi" 
				+ "\tX_hRi\tT_cRo\tX_cRo\tT_xRo\tW_xRo\tdelta1\tdelta2");
		while (k < max && (delta1 > 0.001 || delta2 > 1E-5)){
			// Tubera de K a D
			setN_cDi(KD.getNO());
			// Salida del lquido desecante del deshumidificador
			setN_hDo(D.getSO());
			// Se omite la bomba: De deshumidificador a intercambiador A
			P.setH(PA.getNI().getPressure() - DP.getNO().getPressure());
			setN_hPi(DP.getNO());
			// Tubera de bomba P a Intercambiador A
			setN_hPo(P.getNO());
			// Intercambiador A
			setN_hAi(PA.getNO());
			// Intercambiador A en lnea h
			setN_hAo(A.getSO());
			// Tubera de A a G
			setN_hGi(AG.getNO());
			// Intercambiador G
			setN_hGo(G.getDO());
			// Tubera de G a R
			setN_hRi(GR.getNO());
			// Salida del lquido desecante del regenerador
			setN_cRo(R.getSO());
			// Tubera de R a A
			setN_cAi(RA.getNO());
			// Intercambiador A por la lnea c
			setN_cAo(A.getDO());
			// Tubera de A a K
			setN_cKi(AK.getNO());
			// Intercambiador K
			setN_cKo(K.getSO());
			// Termina el ciclo y se compara la referencia
			System.out.println(String.valueOf(k) + "\t" + String.format("%.2f", getN_tDo().getTem())
				+ "\t" + String.format("%.2f", getN_tDo().getHum() * 1000.0) + "\t" + String.format("%.2f", getN_cDi().getTem())
				+ "\t" + String.format("%.2f", getN_cDi().getXCon() * 1000.0) + "\t" + String.format("%.2f", getN_hDo().getTem()) 
				+ "\t" + String.format("%.2f", getN_hDo().getXCon() * 1000.0) + "\t" + String.format("%.2f", getN_hRi().getTem())
				+ "\t" + String.format("%.2f", getN_hRi().getXCon() * 1000.0) + "\t" + String.format("%.2f", getN_cRo().getTem())
				+ "\t" + String.format("%.2f", getN_cRo().getXCon() * 1000.0) + "\t" + String.format("%.2f", getN_xRo().getTem())
				+ "\t" + String.format("%.2f", getN_xRo().getHum() * 1000.0) + "\t" + String.format("%.3f", delta1)
				+ "\t" + String.format("%.5f", delta2));
			k++;
			delta1 = Math.abs(getN_cKi().getTem() - ref.getTem());
			delta2 = Math.abs(getN_cKi().getXCon() - ref.getXCon());
			ref.setTem(getN_cKi().getTem());
			ref.setXCon(getN_cKi().getXCon());
		}
		return (k < max) ? true : false;
		} catch(OutOfRangeException ex){
			System.out.println(ex.getMessage());
			return false;
		}
	}
	
	public static void main(String[] args){
		// Input parameters
		try{
		Sim3 sim = new Sim3();
		sim.setN_tDi(new Node(new Air(), 20.0, 0.01 / 1.01, 5.0 / 3600.0));
		sim.setN_xRi(new Node(new Air(), 25.0, 0.01 / 1.01, 5.0 / 3600.0));
		sim.setN_wKi(new Node(new Water(), 15.0, 0.0, 10.0 / 3600.0));
		sim.setN_sGi(new Node(new Water(), 70.0, 0.0, 10.0 / 3600.0));
		sim.setN_cKi(new Node(new Desiccant(), 15.0, 0.1, 10.0 / 3600.0));
		sim.setEp_A(0.8);
		sim.setEp_K(0.8);
		sim.setEp_G(0.8);
		sim.setEp_D(0.8, 0.8);
		sim.setEp_R(0.8, 0.8);
		boolean result = sim.run();
		if (result){
			System.out.println("Simulation suceeded!");
		} else {
			System.out.println("Simulation failed");
		}
		} catch(OutOfRangeException ex){
			ex.getMessage();
		}
	}
}

